

#define waterfog
varying vec2 texcoord;

float GetBorderFogMixFactorCylinder(in vec3 eyePlayerPos, in float far, in float z0)
{

    float eyeDist = length(eyePlayerPos.xz);
    float borderFogFactor = smoothstep(far * 0.75, far, eyeDist);
    if (z0 >= 1)
        borderFogFactor = 1;
    return borderFogFactor;
}

float getRawDepth(vec2 uv)
{
    return texture(dhDepthTex0, uv).x;
}

vec3 viewSpacePosAtScreenUV(vec2 uv)
{
    vec3 viewSpaceRay = (dhProjectionInverse * vec4(uv * 2.0 - 1.0, 1.0, 1.0) * near).xyz;
    float rawDepth = getRawDepth(uv);
    return viewSpaceRay * (ld(rawDepth));
}

vec3 viewNormalAtPixelPosition2(vec2 vpos)
{
    // screen uv from vpos
    vec2 uv = vpos * texelSize.xy;

    // current pixel's depth
    float c = getRawDepth(uv);

    // get current pixel's view space position
    vec3 viewSpacePos_c = viewSpacePosAtScreenUV(uv);

    // get view space position at 1 pixel offsets in each major direction
    vec3 viewSpacePos_l = viewSpacePosAtScreenUV(uv + vec2(-1.0, 0.0) * texelSize.xy);
    vec3 viewSpacePos_r = viewSpacePosAtScreenUV(uv + vec2(1.0, 0.0) * texelSize.xy);
    vec3 viewSpacePos_d = viewSpacePosAtScreenUV(uv + vec2(0.0, -1.0) * texelSize.xy);
    vec3 viewSpacePos_u = viewSpacePosAtScreenUV(uv + vec2(0.0, 1.0) * texelSize.xy);

    // get the difference between the current and each offset position
    vec3 l = viewSpacePos_c - viewSpacePos_l;
    vec3 r = viewSpacePos_r - viewSpacePos_c;
    vec3 d = viewSpacePos_c - viewSpacePos_d;
    vec3 u = viewSpacePos_u - viewSpacePos_c;

    // get depth values at 1 & 2 pixels offsets from current along the horizontal
    // axis
    vec4 H = vec4(getRawDepth(uv + vec2(-1.0, 0.0) * texelSize.xy), getRawDepth(uv + vec2(1.0, 0.0) * texelSize.xy),
                  getRawDepth(uv + vec2(-2.0, 0.0) * texelSize.xy), getRawDepth(uv + vec2(2.0, 0.0) * texelSize.xy));

    // get depth values at 1 & 2 pixels offsets from current along the vertical
    // axis
    vec4 V = vec4(getRawDepth(uv + vec2(0.0, -1.0) * texelSize.xy), getRawDepth(uv + vec2(0.0, 1.0) * texelSize.xy),
                  getRawDepth(uv + vec2(0.0, -2.0) * texelSize.xy), getRawDepth(uv + vec2(0.0, 2.0) * texelSize.xy));

    // current pixel's depth difference from slope of offset depth samples
    // differs from original article because we're using non-linear depth values
    // see article's comments
    vec2 he = abs((2 * H.xy - H.zw) - c);
    vec2 ve = abs((2 * V.xy - V.zw) - c);

    // pick horizontal and vertical diff with the smallest depth difference from
    // slopes
    vec3 hDeriv = he.x < he.y ? l : r;
    vec3 vDeriv = ve.x < ve.y ? d : u;

    // get view space normal from the cross product of the best derivatives
    vec3 viewNormal = normalize(cross(hDeriv, vDeriv));

    return viewNormal;
}

float RT(vec3 dir, vec3 position, float noise)
{

    vec3 clipPosition = toClipSpace3DH(position);

    float rayLength = ((position.z + dir.z * sqrt(3.0) * dhFarPlane) > -sqrt(3.0) * dhNearPlane) ? (-sqrt(3.0) * dhNearPlane - position.z) / dir.z : sqrt(3.0) * dhFarPlane;

    vec3 end = toClipSpace3DH(position + dir * rayLength);
    vec3 direction = end - clipPosition;
    float len = max(abs(direction.x) / texelSize.x, abs(direction.y) / texelSize.y);

    vec3 stepv = direction / len;

    float scaledNoise = 24.0 * noise + 1.0;

    vec3 scaledStep = stepv * scaledNoise;

    vec3 spos = clipPosition + scaledStep;

    spos.xy += offsets[framemod8] * texelSize * 0.5;

    float sp = ld(texelFetch(dhDepthTex0, ivec2(spos.xy / texelSize), 0).r);

    float currZ = ld(spos.z);
    float out1 = 1.0 - float((sp < currZ && distance(sp, currZ) / currZ < noise * 0.2));

    return out1;
}

vec2 R2_samples(int n)
{
    vec2 alpha = vec2(0.75487765, 0.56984026);
    return fract(alpha * n);
}
vec3 GenerateUnitVector(vec2 xy)
{
    xy.x *= radians(360.0);
    xy.y = xy.y * 2.0 - 1.0;
    float s = sin(xy.x);
    float c = cos(xy.x);
    return vec3(vec2(s, c) * sqrt(1.0 - xy.y * xy.y), xy.y);
}

void main()
{

    /* DRAWBUFFERS:0 */

    float z = texture(depthtex0, texcoord).x;

    float z0DH = texture(dhDepthTex0, texcoord).x;
    vec3 DH = texture(colortex10, texcoord).rgb;
    vec3 sky = texture(colortex8, texcoord).rgb;

    if (z0DH >= 1.0)
    {
        DH = sky;
    }

    vec3 fragpos = toScreenSpace(vec3(texcoord, z));

    vec3 p3 = mat3(gbufferModelViewInverse) * fragpos + gbufferModelViewInverse[3].xyz;

    float mixfactor = GetBorderFogMixFactorCylinder(p3, far, z);
    vec3 color = texture(colortex0, texcoord.xy).rgb;

    if (mixfactor > 0 && z0DH < 1.0)
    {
        vec3 fragposDH = toScreenSpaceDH(vec3(texcoord, z0DH));

        vec3 viewNormal = viewNormalAtPixelPosition2(gl_FragCoord.xy);
        vec3 worldnormal = mat3(gbufferModelViewInverse) * viewNormal;

        int samples = 4;
        float occlusion = 0.0;
        for (int i = 0; i < samples; i++)
        {
            int seed = frameCounter * int(samples) + i;
            vec2 ij = fract(R2_samples(seed) + blueNoise4(gl_FragCoord.xy).rg);
            vec3 L = normalize(worldnormal + GenerateUnitVector(ij));

            occlusion += RT(mat3(gbufferModelView) * L, fragposDH.xyz, blueNoise(gl_FragCoord.xy));
        }

        DH *= pow(occlusion / float(samples), 2.0);
    }

    color.rgb = mix(color.rgb, DH, mixfactor);

    gl_FragData[0].rgb = vec3(color);
}
