#extension GL_ARB_gpu_shader5 : enable
// data1

flat varying vec3 ambientUp;
flat varying vec3 ambientDown;
flat varying vec3 ambientLeft;
flat varying vec3 ambientRight;
flat varying vec3 ambientB;
flat varying vec3 ambientF;
flat varying float ambientA;
varying vec2 texcoord;

#include "/lib/light.glsl"

vec2 tapLocation(int sampleNumber, float spinAngle, int nb, float nbRot)
{
    float alpha = float(sampleNumber + spinAngle / 6.28) / nb;
    float angle = alpha * (nbRot * 6.28) + spinAngle;

    float ssR = alpha * alpha;
    float sin_v, cos_v;

    sin_v = sin(angle);
    cos_v = cos(angle);

    return vec2(cos_v, sin_v) * ssR;
}
#if SSR_QUALITY == 2
    #include "/lib/reflection_lib_fast.glsl"
#endif
#if SSR_QUALITY == 3
    #include "/lib/reflection_lib.glsl"

#endif

vec3 Reproject(vec3 position)
{
    bool isHand = position.z < 0.6;
    if (position.z < 1.0 && !isHand)
    {
        position = ViewSpaceFromScreenSpace(position, gbufferProjectionInverse);
        position = mat3(gbufferModelViewInverse) * position + gbufferModelViewInverse[3].xyz;
        position += (cameraPosition - previousCameraPosition);
        position = mat3(gbufferPreviousModelView) * position + gbufferPreviousModelView[3].xyz;
        position = ScreenSpaceFromViewSpace(position, gbufferPreviousProjection);
    }
    else
    {
        position = ViewSpaceFromScreenSpace(position, gbufferProjectionInverse);
        position = mat3(gbufferModelViewInverse) * position + gbufferModelViewInverse[3].xyz;
        position = mat3(gbufferPreviousModelView) * position + gbufferPreviousModelView[3].xyz;
        position = ScreenSpaceFromViewSpace(position, gbufferPreviousProjection);
    }

    return position;
}
float MinDepth3x3(vec2 uv)
{
    ivec2 fragCoord = ivec2(gl_FragCoord.xy);
    float depth = texelFetch(depthtex0, fragCoord, 0).x;

    for (int x = -1; x <= 1; ++x)
    {
        for (int y = -1; y <= 1; ++y)
        {
            if (x != 0 || y != 0)
            {
                depth = min(depth, texelFetch(depthtex0, fragCoord + ivec2(x, y), 0).x);
            }
        }
    }

    return depth;
}

void main()
{

#ifdef SSR_ACCUMULATION
    /* RENDERTARGETS: 12 */
#else
    /* RENDERTARGETS: 0,1 */
#endif

    vec2 texSize = viewResolution / textureSize(colortex12, 0); // Get the size of the base mipmap level (LOD 0)

    float z = texture(depthtex0, texcoord).x;
    bool hand = texture(depthtex2, texcoord).x > z;
#ifdef TAA
    vec2 tempOffset = offsets[framemod8] * texelSize * 0.5;
    vec3 fragpos = toScreenSpace(vec3(texcoord - tempOffset, z));
#else
    vec3 fragpos = toScreenSpace(vec3(texcoord, z));
#endif

    float materials = texelFetch(colortex3, ivec2(gl_FragCoord.xy * texSize), 0).x;
    vec4 normalTex = texelFetch(colortex2, ivec2(gl_FragCoord.xy * texSize), 0);

    vec2 decoded = decodeVec2(normalTex.a);
    vec2 decoded2 = decodeVec2(materials);
    float reflectivity = decoded2.x;
    float smoothness = decoded2.y;
    vec3 scene = texture(colortex0, texcoord).rgb;
#ifdef SSR_ACCUMULATION
    if (z >= 1 || smoothness < REFLECTION_CUTOFF)
    {
        gl_FragData[0].rgb = vec3(0.0);
        return;
    }
#else
    if (z >= 1 || smoothness < REFLECTION_CUTOFF)
    {
        gl_FragData[0].rgb = scene;
        return;
    }
#endif
    vec4 reflections = vec4(0);

    vec3 normal = (decodeNormal(normalTex.rg));
    vec2 lightmap = decodeVec2(normalTex.b);

    vec3 f0 = vec3(reflectivity);
    reflections = computeReflections(lightmap.y, smoothness, f0, normal, fragpos, decoded.y, scene, hand);
    gl_FragData[0] = reflections;

#ifndef SSR_ACCUMULATION

    gl_FragData[1] = vec4(0.0);

#endif
}
