// Varyings

flat varying vec2 midtex;
flat varying vec4 labvalue;
flat varying float emissive;

varying vec4 lmtexcoord;
varying vec4 color;

#if defined NORMAL_MAPS || defined POM
varying vec4 vtexcoordam;
varying vec4 vtexcoord;
varying vec4 tangent;
varying vec4 normalMat2;
#endif
varying vec3 normalMat;

attribute vec4 mc_Entity;
attribute vec4 mc_midTexCoord;
attribute vec4 at_tangent;

#define diagonal3(m) vec3(m[0].x, m[1].y, m[2].z)
#define projMAD(m, v) (diagonal3(m) * (v) + m[3].xyz)

#include "/lib/vertex/main.glsl"

float computeEmissive(float lmtexcoordZ, int entityCheck)
{
    return (entityCheck >= 100 && entityCheck <= 355)
               ? pow(lmtexcoordZ, 8.0)
               : 0.0;
}

#if defined NORMAL_MAPS || defined POM
mat3 computeTBN(vec3 normal)
{
    return computeTBNMatrix(gbufferModelViewInverse, gl_NormalMatrix, at_tangent, gl_Normal);
}
#endif

vec3 computePosition(vec4 vertex)
{
    return (mat3(gl_ModelViewMatrix) * vertex.xyz) + gl_ModelViewMatrix[3].xyz;
}

void main()
{
    lmtexcoord = vec4(gl_MultiTexCoord0.xy, gl_MultiTexCoord1.xy * recip);

    int entityCheck = int(mc_Entity.x);
    labvalue = getMaterialProperties(entityCheck);

    emissive = computeEmissive(lmtexcoord.z, entityCheck);

    color = gl_Color;

    midtex = mc_midTexCoord.st;

#if defined NORMAL_MAPS || defined POM || defined COMPLEX_NORMALS
    vec3 normTransformed = normalize(gl_NormalMatrix * gl_Normal);
#endif

#if defined COMPLEX_NORMALS || defined POM
    normalMat = viewToWorld(normTransformed);
#else
    normalMat = gl_Normal;
#endif

#if defined NORMAL_MAPS || defined POM
    vec2 midcoord = (gl_TextureMatrix[0] * mc_midTexCoord).st;
    normalMat2 = vec4(normTransformed, 0.0);

    bool tangentValid = dot(at_tangent.xyz, at_tangent.xyz) > 1e-9;
    vec3 T = tangentValid
                 ? normalize(mat3(gl_NormalMatrix) * at_tangent.xyz)
                 : vec3(0.0);
    tangent = vec4(T, at_tangent.w);

    vec2 diff = lmtexcoord.xy - midcoord;
    vtexcoordam.pq = abs(diff) * 2.0;
    vtexcoordam.st = min(lmtexcoord.xy, 2.0 * midcoord - lmtexcoord.xy);
    vtexcoord.xy = sign(diff) * 0.5 + 0.5;
#endif
    vec3 pos = computePosition(gl_Vertex);

#ifdef LEAVES

    bool Flower = isFlower(entityCheck, (gl_MultiTexCoord0.t < mc_midTexCoord.t));
    bool LeavesFlag = isLeaves(entityCheck, (abs(pos.z) < 128.0 && gl_MultiTexCoord1.y > 100.0));
    if (Flower || LeavesFlag)
    {
        pos = computeWaveOffset(pos, cameraPosition, gl_Vertex, gl_MultiTexCoord1, gbufferModelView);
    }
#endif

#if defined ENTITY && defined NO_ENTITY_TAA
    gl_Position = toClipSpace4(pos);
#else
    #ifdef LEAVES
    gl_Position = taaFunc(toClipSpace4(pos));
    #else
    vec4 viewPos = gl_ModelViewMatrix * gl_Vertex;
    gl_Position = taaFunc(vec4(diagonal3(gl_ProjectionMatrix) * viewPos.xyz + gl_ProjectionMatrix[3].xyz, -viewPos.z));

    #endif
#endif
}
